@@ -29,6 +29,10 @@
  *   and #ClutterActor:scale-y instead.
  *
  * A #ClutterBehaviourScale interpolates actors size between two values.
+ *
+ * Deprecated: 1.6: Use the #ClutterActor:scale-x and #ClutterActor:scale-y
+ *   properties, and clutter_actor_animate(), or #ClutterAnimator or
+ *   #ClutterState instead.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,7 +41,6 @@
 
 #include <math.h>
 
-#undef CLUTTER_DISABLE_DEPRECATED
 #include "clutter-behaviour-scale.h"
 
 #include "clutter-debug.h"
@@ -367,25 +370,25 @@ clutter_behaviour_scale_set_bounds (ClutterBehaviourScale *scale,
   if (priv->x_scale_start != x_scale_start)
     {
       priv->x_scale_start = x_scale_start;
-      _clutter_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_X_SCALE_START]);
+      g_object_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_X_SCALE_START]);
     }
 
   if (priv->y_scale_start != y_scale_start)
     {
       priv->y_scale_start = y_scale_start;
-      _clutter_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_Y_SCALE_START]);
+      g_object_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_Y_SCALE_START]);
     }
 
   if (priv->x_scale_end != x_scale_end)
     {
       priv->x_scale_end = x_scale_end;
-      _clutter_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_X_SCALE_END]);
+      g_object_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_X_SCALE_END]);
     }
 
   if (priv->y_scale_end != y_scale_end)
     {
       priv->y_scale_end = y_scale_end;
-      _clutter_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_Y_SCALE_END]);
+      g_object_notify_by_pspec (G_OBJECT (scale), obj_props[PROP_Y_SCALE_END]);
     }
 
   g_object_thaw_notify (G_OBJECT (scale));
