@@ -35,6 +35,7 @@
 #include "clutter-actor-private.h"
 #include "clutter-paint-volume-private.h"
 #include "clutter-private.h"
+#include "clutter-stage-private.h"
 
 G_DEFINE_BOXED_TYPE (ClutterPaintVolume, clutter_paint_volume,
                      clutter_paint_volume_copy,
@@ -90,11 +91,9 @@ _clutter_paint_volume_new (ClutterActor *actor)
  * free it during _paint_volume_free().
  */
 void
-_clutter_paint_volume_init_static (ClutterActor *actor,
-                                   ClutterPaintVolume *pv)
+_clutter_paint_volume_init_static (ClutterPaintVolume *pv,
+                                   ClutterActor *actor)
 {
-  g_return_if_fail (actor != NULL);
-
   pv->actor = actor;
 
   memset (pv->vertices, 0, 8 * sizeof (ClutterVertex));
@@ -144,7 +143,9 @@ void
 _clutter_paint_volume_set_from_volume (ClutterPaintVolume       *pv,
                                        const ClutterPaintVolume *src)
 {
+  gboolean is_static = pv->is_static;
   memcpy (pv, src, sizeof (ClutterPaintVolume));
+  pv->is_static = is_static;
 }
 
 /**
@@ -189,7 +190,6 @@ clutter_paint_volume_set_origin (ClutterPaintVolume  *pv,
   int i;
 
   g_return_if_fail (pv != NULL);
-  g_return_if_fail (pv->is_axis_aligned);
 
   dx = origin->x - pv->vertices[0].x;
   dy = origin->y - pv->vertices[0].y;
@@ -242,7 +242,8 @@ _clutter_paint_volume_update_is_empty (ClutterPaintVolume *pv)
  * @pv: a #ClutterPaintVolume
  * @width: the width of the paint volume, in pixels
  *
- * Sets the width of the paint volume.
+ * Sets the width of the paint volume. The width is measured along
+ * the x axis in the actor coordinates that @pv is associated with.
  *
  * Since: 1.6
  */
@@ -253,7 +254,6 @@ clutter_paint_volume_set_width (ClutterPaintVolume *pv,
   gfloat right_xpos;
 
   g_return_if_fail (pv != NULL);
-  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (width >= 0.0f);
 
   /* If the volume is currently empty then only the origin is
@@ -261,6 +261,9 @@ clutter_paint_volume_set_width (ClutterPaintVolume *pv,
   if (pv->is_empty)
     pv->vertices[1] = pv->vertices[3] = pv->vertices[4] = pv->vertices[0];
 
+  if (!pv->is_axis_aligned)
+    _clutter_paint_volume_axis_align (pv);
+
   right_xpos = pv->vertices[0].x + width;
 
   /* Move the right vertices of the paint box relative to the
@@ -279,9 +282,30 @@ clutter_paint_volume_set_width (ClutterPaintVolume *pv,
  * clutter_paint_volume_get_width:
  * @pv: a #ClutterPaintVolume
  *
- * Retrieves the width set using clutter_paint_volume_get_width()
+ * Retrieves the width of the volume's, axis aligned, bounding box.
  *
- * Return value: the width, in pixels
+ * In other words; this takes into account what actor's coordinate
+ * space @pv belongs too and conceptually fits an axis aligned box
+ * around the volume. It returns the size of that bounding box as
+ * measured along the x-axis.
+ *
+ * <note>If, for example, clutter_actor_get_transformed_paint_volume()
+ * is used to transform a 2D child actor that is 100px wide, 100px
+ * high and 0px deep into container coordinates then the width might
+ * not simply be 100px if the child actor has a 3D rotation applied to
+ * it.
+ *
+ * Remember; after clutter_actor_get_transformed_paint_volume() is
+ * used then a transformed child volume will be defined relative to the
+ * ancestor container actor and so a 2D child actor
+ * can have a 3D bounding volume.</note>
+ *
+ * <note>There are no accuracy guarantees for the reported width,
+ * except that it must always be >= to the true width. This is
+ * because actors may report simple, loose fitting paint-volumes
+ * for efficiency</note>
+
+ * Return value: the width, in units of @pv's local coordinate system.
  *
  * Since: 1.6
  */
@@ -289,10 +313,19 @@ gfloat
 clutter_paint_volume_get_width (const ClutterPaintVolume *pv)
 {
   g_return_val_if_fail (pv != NULL, 0.0);
-  g_return_val_if_fail (pv->is_axis_aligned, 0);
 
   if (pv->is_empty)
     return 0;
+  else if (!pv->is_axis_aligned)
+    {
+      ClutterPaintVolume tmp;
+      float width;
+      _clutter_paint_volume_copy_static (pv, &tmp);
+      _clutter_paint_volume_axis_align (&tmp);
+      width = tmp.vertices[1].x - tmp.vertices[0].x;
+      clutter_paint_volume_free (&tmp);
+      return width;
+    }
   else
     return pv->vertices[1].x - pv->vertices[0].x;
 }
@@ -302,7 +335,8 @@ clutter_paint_volume_get_width (const ClutterPaintVolume *pv)
  * @pv: a #ClutterPaintVolume
  * @height: the height of the paint volume, in pixels
  *
- * Sets the height of the paint volume.
+ * Sets the height of the paint volume. The height is measured along
+ * the y axis in the actor coordinates that @pv is associated with.
  *
  * Since: 1.6
  */
@@ -313,7 +347,6 @@ clutter_paint_volume_set_height (ClutterPaintVolume *pv,
   gfloat height_ypos;
 
   g_return_if_fail (pv != NULL);
-  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (height >= 0.0f);
 
   /* If the volume is currently empty then only the origin is
@@ -321,6 +354,9 @@ clutter_paint_volume_set_height (ClutterPaintVolume *pv,
   if (pv->is_empty)
     pv->vertices[1] = pv->vertices[3] = pv->vertices[4] = pv->vertices[0];
 
+  if (!pv->is_axis_aligned)
+    _clutter_paint_volume_axis_align (pv);
+
   height_ypos = pv->vertices[0].y + height;
 
   /* Move the bottom vertices of the paint box relative to the
@@ -338,10 +374,30 @@ clutter_paint_volume_set_height (ClutterPaintVolume *pv,
  * clutter_paint_volume_get_height:
  * @pv: a #ClutterPaintVolume
  *
- * Retrieves the height of the paint volume set using
- * clutter_paint_volume_get_height()
+ * Retrieves the height of the volume's, axis aligned, bounding box.
  *
- * Return value: the height of the paint volume, in pixels
+ * In other words; this takes into account what actor's coordinate
+ * space @pv belongs too and conceptually fits an axis aligned box
+ * around the volume. It returns the size of that bounding box as
+ * measured along the y-axis.
+ *
+ * <note>If, for example, clutter_actor_get_transformed_paint_volume()
+ * is used to transform a 2D child actor that is 100px wide, 100px
+ * high and 0px deep into container coordinates then the height might
+ * not simply be 100px if the child actor has a 3D rotation applied to
+ * it.
+ *
+ * Remember; after clutter_actor_get_transformed_paint_volume() is
+ * used then a transformed child volume will be defined relative to the
+ * ancestor container actor and so a 2D child actor
+ * can have a 3D bounding volume.</note>
+ *
+ * <note>There are no accuracy guarantees for the reported height,
+ * except that it must always be >= to the true height. This is
+ * because actors may report simple, loose fitting paint-volumes
+ * for efficiency</note>
+ *
+ * Return value: the height, in units of @pv's local coordinate system.
  *
  * Since: 1.6
  */
@@ -349,10 +405,19 @@ gfloat
 clutter_paint_volume_get_height (const ClutterPaintVolume *pv)
 {
   g_return_val_if_fail (pv != NULL, 0.0);
-  g_return_val_if_fail (pv->is_axis_aligned, 0);
 
   if (pv->is_empty)
     return 0;
+  else if (!pv->is_axis_aligned)
+    {
+      ClutterPaintVolume tmp;
+      float height;
+      _clutter_paint_volume_copy_static (pv, &tmp);
+      _clutter_paint_volume_axis_align (&tmp);
+      height = tmp.vertices[3].y - tmp.vertices[0].y;
+      clutter_paint_volume_free (&tmp);
+      return height;
+    }
   else
     return pv->vertices[3].y - pv->vertices[0].y;
 }
@@ -362,7 +427,8 @@ clutter_paint_volume_get_height (const ClutterPaintVolume *pv)
  * @pv: a #ClutterPaintVolume
  * @depth: the depth of the paint volume, in pixels
  *
- * Sets the depth of the paint volume.
+ * Sets the depth of the paint volume. The depth is measured along
+ * the z axis in the actor coordinates that @pv is associated with.
  *
  * Since: 1.6
  */
@@ -373,7 +439,6 @@ clutter_paint_volume_set_depth (ClutterPaintVolume *pv,
   gfloat depth_zpos;
 
   g_return_if_fail (pv != NULL);
-  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (depth >= 0.0f);
 
   /* If the volume is currently empty then only the origin is
@@ -381,6 +446,9 @@ clutter_paint_volume_set_depth (ClutterPaintVolume *pv,
   if (pv->is_empty)
     pv->vertices[1] = pv->vertices[3] = pv->vertices[4] = pv->vertices[0];
 
+  if (!pv->is_axis_aligned)
+    _clutter_paint_volume_axis_align (pv);
+
   depth_zpos = pv->vertices[0].z + depth;
 
   /* Move the back vertices of the paint box relative to the
@@ -399,10 +467,30 @@ clutter_paint_volume_set_depth (ClutterPaintVolume *pv,
  * clutter_paint_volume_get_depth:
  * @pv: a #ClutterPaintVolume
  *
- * Retrieves the depth of the paint volume set using
- * clutter_paint_volume_get_depth()
+ * Retrieves the depth of the volume's, axis aligned, bounding box.
  *
- * Return value: the depth
+ * In other words; this takes into account what actor's coordinate
+ * space @pv belongs too and conceptually fits an axis aligned box
+ * around the volume. It returns the size of that bounding box as
+ * measured along the z-axis.
+ *
+ * <note>If, for example, clutter_actor_get_transformed_paint_volume()
+ * is used to transform a 2D child actor that is 100px wide, 100px
+ * high and 0px deep into container coordinates then the depth might
+ * not simply be 0px if the child actor has a 3D rotation applied to
+ * it.
+ *
+ * Remember; after clutter_actor_get_transformed_paint_volume() is
+ * used then the transformed volume will be defined relative to the
+ * container actor and in container coordinates a 2D child actor
+ * can have a 3D bounding volume.</note>
+ *
+ * <note>There are no accuracy guarantees for the reported depth,
+ * except that it must always be >= to the true depth. This is
+ * because actors may report simple, loose fitting paint-volumes
+ * for efficiency.</note>
+ *
+ * Return value: the depth, in units of @pv's local coordinate system.
  *
  * Since: 1.6
  */
@@ -410,10 +498,19 @@ gfloat
 clutter_paint_volume_get_depth (const ClutterPaintVolume *pv)
 {
   g_return_val_if_fail (pv != NULL, 0.0);
-  g_return_val_if_fail (pv->is_axis_aligned, 0);
 
   if (pv->is_empty)
     return 0;
+  else if (!pv->is_axis_aligned)
+    {
+      ClutterPaintVolume tmp;
+      float depth;
+      _clutter_paint_volume_copy_static (pv, &tmp);
+      _clutter_paint_volume_axis_align (&tmp);
+      depth = tmp.vertices[4].z - tmp.vertices[0].z;
+      clutter_paint_volume_free (&tmp);
+      return depth;
+    }
   else
     return pv->vertices[4].z - pv->vertices[0].z;
 }
@@ -424,8 +521,10 @@ clutter_paint_volume_get_depth (const ClutterPaintVolume *pv)
  *      union
  * @another_pv: A second #ClutterPaintVolume to union with @pv
  *
- * Updates the geometry of @pv to be the union bounding box that
- * encompases @pv and @another_pv.
+ * Updates the geometry of @pv to encompass @pv and @another_pv.
+ *
+ * <note>There are no guarantees about how precisely the two volumes
+ * will be encompassed.</note>
  *
  * Since: 1.6
  */
@@ -433,12 +532,13 @@ void
 clutter_paint_volume_union (ClutterPaintVolume *pv,
                             const ClutterPaintVolume *another_pv)
 {
-  static const int key_vertices[4] = { 0, 1, 3, 4 };
+  ClutterPaintVolume aligned_pv;
 
   g_return_if_fail (pv != NULL);
-  g_return_if_fail (pv->is_axis_aligned);
   g_return_if_fail (another_pv != NULL);
-  g_return_if_fail (another_pv->is_axis_aligned);
+
+  /* Both volumes have to belong to the same local coordinate space */
+  g_return_if_fail (pv->actor == another_pv->actor);
 
   /* NB: we only have to update vertices 0, 1, 3 and 4
    * (See the ClutterPaintVolume typedef for more details) */
@@ -452,13 +552,20 @@ clutter_paint_volume_union (ClutterPaintVolume *pv,
 
   if (pv->is_empty)
     {
-      int i;
-      for (i = 0; i < 4; i++)
-        pv->vertices[key_vertices[i]] = another_pv->vertices[key_vertices[i]];
-      pv->is_2d = another_pv->is_2d;
+      _clutter_paint_volume_set_from_volume (pv, another_pv);
       goto done;
     }
 
+  if (!pv->is_axis_aligned)
+    _clutter_paint_volume_axis_align (pv);
+
+  if (!another_pv->is_axis_aligned)
+    {
+      _clutter_paint_volume_copy_static (another_pv, &aligned_pv);
+      _clutter_paint_volume_axis_align (&aligned_pv);
+      another_pv = &aligned_pv;
+    }
+
   /* grow left*/
   /* left vertices 0, 3, 4, 7 */
   if (another_pv->vertices[0].x < pv->vertices[0].x)
@@ -543,33 +650,45 @@ done:
 void
 _clutter_paint_volume_complete (ClutterPaintVolume *pv)
 {
-  if (pv->is_complete || pv->is_empty)
+  float dx_l2r, dy_l2r, dz_l2r;
+  float dx_t2b, dy_t2b, dz_t2b;
+
+  if (pv->is_empty)
     return;
 
-  /* TODO: it is possible to complete non axis aligned volumes too. */
-  g_return_if_fail (pv->is_axis_aligned);
+  /* Find the vector that takes us from any vertex on the left face to
+   * the corresponding vertex on the right face. */
+  dx_l2r = pv->vertices[1].x - pv->vertices[0].x;
+  dy_l2r = pv->vertices[1].y - pv->vertices[0].y;
+  dz_l2r = pv->vertices[1].z - pv->vertices[0].z;
+
+  /* Find the vector that takes us from any vertex on the top face to
+   * the corresponding vertex on the bottom face. */
+  dx_t2b = pv->vertices[3].x - pv->vertices[0].x;
+  dy_t2b = pv->vertices[3].y - pv->vertices[0].y;
+  dz_t2b = pv->vertices[3].z - pv->vertices[0].z;
 
   /* front-bottom-right */
-  pv->vertices[2].x = pv->vertices[1].x;
-  pv->vertices[2].y = pv->vertices[3].y;
-  pv->vertices[2].z = pv->vertices[0].z;
+  pv->vertices[2].x = pv->vertices[3].x + dx_l2r;
+  pv->vertices[2].y = pv->vertices[3].y + dy_l2r;
+  pv->vertices[2].z = pv->vertices[3].z + dz_l2r;
 
   if (G_UNLIKELY (!pv->is_2d))
     {
       /* back-top-right */
-      pv->vertices[5].x = pv->vertices[1].x;
-      pv->vertices[5].y = pv->vertices[0].y;
-      pv->vertices[5].z = pv->vertices[4].z;
+      pv->vertices[5].x = pv->vertices[4].x + dx_l2r;
+      pv->vertices[5].y = pv->vertices[4].y + dy_l2r;
+      pv->vertices[5].z = pv->vertices[4].z + dz_l2r;
 
       /* back-bottom-right */
-      pv->vertices[6].x = pv->vertices[1].x;
-      pv->vertices[6].y = pv->vertices[3].y;
-      pv->vertices[6].z = pv->vertices[4].z;
+      pv->vertices[6].x = pv->vertices[5].x + dx_t2b;
+      pv->vertices[6].y = pv->vertices[5].y + dy_t2b;
+      pv->vertices[6].z = pv->vertices[5].z + dz_t2b;
 
       /* back-bottom-left */
-      pv->vertices[7].x = pv->vertices[0].x;
-      pv->vertices[7].y = pv->vertices[3].y;
-      pv->vertices[7].z = pv->vertices[4].z;
+      pv->vertices[7].x = pv->vertices[4].x + dx_t2b;
+      pv->vertices[7].y = pv->vertices[4].y + dy_t2b;
+      pv->vertices[7].z = pv->vertices[4].z + dz_t2b;
     }
 
   pv->is_complete = TRUE;
@@ -651,7 +770,7 @@ void
 _clutter_paint_volume_project (ClutterPaintVolume *pv,
                                const CoglMatrix *modelview,
                                const CoglMatrix *projection,
-                               const int *viewport)
+                               const float *viewport)
 {
   int transform_count;
 
@@ -747,8 +866,6 @@ _clutter_paint_volume_axis_align (ClutterPaintVolume *pv)
   if (pv->is_empty)
     return;
 
-  g_return_if_fail (pv->is_complete);
-
   if (G_LIKELY (pv->is_axis_aligned))
     return;
 
@@ -760,6 +877,9 @@ _clutter_paint_volume_axis_align (ClutterPaintVolume *pv)
       return;
     }
 
+  if (!pv->is_complete)
+    _clutter_paint_volume_complete (pv);
+
   origin = pv->vertices[0];
   max_x = pv->vertices[0].x;
   max_y = pv->vertices[0].y;
@@ -909,3 +1029,120 @@ _clutter_paint_volume_set_reference_actor (ClutterPaintVolume *pv,
 
   pv->actor = actor;
 }
+
+ClutterCullResult
+_clutter_paint_volume_cull (ClutterPaintVolume *pv,
+                            const ClutterPlane *planes)
+{
+  int vertex_count;
+  ClutterVertex *vertices = pv->vertices;
+  gboolean partial = FALSE;
+  int i;
+  int j;
+
+  if (pv->is_empty)
+    return CLUTTER_CULL_RESULT_OUT;
+
+  /* We expect the volume to already be transformed into eye coordinates
+   */
+  g_return_val_if_fail (pv->is_complete == TRUE, CLUTTER_CULL_RESULT_IN);
+  g_return_val_if_fail (pv->actor == NULL, CLUTTER_CULL_RESULT_IN);
+
+  /* Most actors are 2D so we only have to transform the front 4
+   * vertices of the paint volume... */
+  if (G_LIKELY (pv->is_2d))
+    vertex_count = 4;
+  else
+    vertex_count = 8;
+
+  for (i = 0; i < 4; i++)
+    {
+      int out = 0;
+      for (j = 0; j < vertex_count; j++)
+        {
+          ClutterVertex p;
+          float distance;
+
+          /* XXX: for perspective projections this can be optimized
+           * out because all the planes should pass through the origin
+           * so (0,0,0) is a valid v0. */
+          p.x = vertices[j].x - planes[i].v0.x;
+          p.y = vertices[j].y - planes[i].v0.y;
+          p.z = vertices[j].z - planes[i].v0.z;
+
+          distance =
+            planes[i].n.x * p.x + planes[i].n.y * p.y + planes[i].n.z * p.z;
+
+          if (distance < 0)
+            out++;
+        }
+
+      if (out == vertex_count)
+        return CLUTTER_CULL_RESULT_OUT;
+      else if (out != 0)
+        partial = TRUE;
+    }
+
+  if (partial)
+    return CLUTTER_CULL_RESULT_PARTIAL;
+  else
+    return CLUTTER_CULL_RESULT_IN;
+}
+
+void
+_clutter_paint_volume_get_stage_paint_box (ClutterPaintVolume *pv,
+                                           ClutterStage *stage,
+                                           ClutterActorBox *box)
+{
+  ClutterPaintVolume projected_pv;
+  CoglMatrix modelview;
+  CoglMatrix projection;
+  float viewport[4];
+
+  _clutter_paint_volume_copy_static (pv, &projected_pv);
+
+  cogl_matrix_init_identity (&modelview);
+
+  /* If the paint volume isn't already in eye coordinates... */
+  if (pv->actor)
+    _clutter_actor_apply_relative_transformation_matrix (pv->actor, NULL,
+                                                         &modelview);
+
+  _clutter_stage_get_projection_matrix (stage, &projection);
+  _clutter_stage_get_viewport (stage,
+                               &viewport[0],
+                               &viewport[1],
+                               &viewport[2],
+                               &viewport[3]);
+
+  _clutter_paint_volume_project (&projected_pv,
+                                 &modelview,
+                                 &projection,
+                                 viewport);
+
+  _clutter_paint_volume_get_bounding_box (&projected_pv, box);
+  clutter_actor_box_clamp_to_pixel (box);
+
+  clutter_paint_volume_free (&projected_pv);
+}
+
+void
+_clutter_paint_volume_transform_relative (ClutterPaintVolume *pv,
+                                          ClutterActor *relative_to_ancestor)
+{
+  CoglMatrix matrix;
+  ClutterActor *actor;
+
+  actor = pv->actor;
+
+  g_return_if_fail (actor != NULL);
+
+  _clutter_paint_volume_set_reference_actor (pv, relative_to_ancestor);
+
+  cogl_matrix_init_identity (&matrix);
+  _clutter_actor_apply_relative_transformation_matrix (actor,
+                                                       relative_to_ancestor,
+                                                      &matrix);
+
+  _clutter_paint_volume_transform (pv, &matrix);
+}
